(function() {
    "use strict";

    angular.module("weChatApp")
        .controller("dashboardCtrl", ["$scope", "$timeout", "$q", "_currentUser", "_unreachableObjects", "DashboardHelper",
            "TranslationService", "SentMessageService", "TerritoryService", "ConsentService", "Utility",
            "ChartJsPlugins", "AppConstants", "Queries", DashboardCtrl])
    ;

    function DashboardCtrl($scope, $timeout, $q, _currentUser, _unreachableObjects, dashboardHelper,
                           translationService, sentMessageService, territoryService, consentService, utility,
                           chartJsPlugins, appConstants, queries) {
        var dash = this;
        var OBJECT_LABELS = {};
        var FIELD_LABELS = {};
        var PICKLIST_LABELS = {};
        var user_timezone;
        var user_locale;

        dashboardHelper.excludeUnreachableFieldFromQueries(queries, _unreachableObjects);
        dashboardHelper.getLabels(_unreachableObjects).then(function(resp) {
            OBJECT_LABELS = resp.objectLabels;
            FIELD_LABELS = resp.fieldLabels;
            PICKLIST_LABELS = resp.picklistLabels;
            preStartSetup();
            start();
        });

        function preStartSetup() {
            user_timezone = _currentUser.TimeZoneSidKey;
            user_locale = _currentUser.LocaleSidKey;

            if(!moment.tz.zone(user_timezone)) {
                user_timezone = "GMT";
            }

            moment.locale("en");
            if(user_locale) {
                moment.locale(user_locale); // moment will ignore the locale here if it's not valid
            }

            dash.detailMode = false;
        }

        function start() {
            initGlobalDateFilter();
            var filterDates = getFilterDates();

            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(filterDates);
            }
            if(dashboardHelper.okToEnableTile(appConstants.CONSENT, _unreachableObjects)) {
                getConsents(filterDates);
            }
        }

        function initGlobalDateFilter() {
            dash.globalDateFilterOptions = [];
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 30]),
                dayRange: -30
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 60]),
                dayRange: -60
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 90]),
                dayRange: -90
            });
            dash.selectedDateFilter = dash.globalDateFilterOptions[0];
        }

        function getFilterDates() {
            var backward = dash.selectedDateFilter.dayRange !== Math.abs(dash.selectedDateFilter.dayRange);
            var dayRange = Math.abs(dash.selectedDateFilter.dayRange);

            var filterEndDate;
            var filterStartDate;
            if(backward) {
                filterEndDate = moment(); // now
                filterStartDate = filterEndDate.clone().subtract(dayRange, "days");
            } else {
                filterStartDate = moment(); // now
                filterEndDate = filterStartDate.clone().add(dayRange, "days");
            }

            var startDateUTC = moment.utc(filterStartDate);
            var endDateUTC = moment.utc(filterEndDate);
            return {
                startDatetime: startDateUTC.toISOString(),
                endDatetime: endDateUTC.toISOString(),
                startDate: startDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                endDate: endDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                startDateMoment: startDateUTC,
                endDateMoment: endDateUTC
            };
        }

        function getSentMessages(filterDates) {
            var includeAccountName = true;
            var includeMultichannelContent = true;
            var includeContentType = true;
            if(!!_unreachableObjects.Sent_Message_vod__c) {
                includeAccountName = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Account"));
                includeMultichannelContent = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Multichannel_Content_vod__c"));
                includeContentType = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Content_Type_vod__c"));
            }

            sentMessageService.getSentMessageByUserId(_currentUser.Id, filterDates, includeAccountName,
                includeMultichannelContent, includeContentType).then(function(sentMessageResp) {
                console.log(sentMessageResp);

                var openedSMs = sentMessageResp.filter(function(sm) {
                    // for WM discrepancy
                    if(typeof sm.Opened_vod__c === "string") {
                        return !!parseInt(sm.Opened_vod__c);
                    }
                    return !!sm.Opened_vod__c;
                });
                var clickedSMs = openedSMs.filter(function(sm) {
                    // for WM discrepancy
                    if(typeof sm.Content_Clicked_vod__c === "string") {
                        return !!parseInt(sm.Content_Clicked_vod__c);
                    }
                    return !!sm.Content_Clicked_vod__c;
                });

                dash.sentMessageResp = sentMessageResp;
                dash.messageOpenRate = 0;
                dash.clickthroughRate = 0;
                if(sentMessageResp.length > 0) {
                    dash.messageOpenRate = (100 * openedSMs.length / sentMessageResp.length).toFixed();
                }
                if(openedSMs.length > 0) {
                    dash.clickthroughRate = (100 * clickedSMs.length / openedSMs.length).toFixed();
                }
                dash.sentMessageReady = true;
            });
        }

        function getConsents(filterDates) {
            territoryService.getAccountsUnderUserTerritory(_currentUser.Id).then(function(accountResp) {
                console.log(accountResp);
                consentService.getWeChatConsentByAccounts(accountResp, filterDates).then(function(consentResp) {
                    console.log(consentResp);

                    /** To be considered qualified consent, the record need to
                     *  1. Opt_Type_vod__c = Opt_In_vod
                     *  2. Channel_Value_vod__c is not empty
                     *  3. Opt_Expiration_Date_vod__c >= TODAY OR EMPTY
                     */
                    var activeOptInConsent = consentResp.filter(function(c) {
                        return c.Opt_Type_vod__c === "Opt_In_vod"
                            && !!c.Channel_Value_vod__c
                            && (!c.Opt_Expiration_Date_vod__c || utility.parseStringToDateValue(c.Opt_Expiration_Date_vod__c) >= moment.utc().valueOf());
                    });
                    var targetAccountsWithActiveOptInConsent = _.uniqBy(activeOptInConsent, "Account_vod__c");
                    var targetAccountsWithSentMessage = _.uniqBy(dash.sentMessageResp, "Account_vod__c").filter(function(sm) {
                        // target accounts must must match account from the right territory
                        return !!_.find(accountResp, function(account) {
                            return account.Id === sm.Account_vod__c;
                        });
                    });

                    dash.consentResp = consentResp;
                    dash.hcpOptInPercentage = 0;
                    dash.hcpSentMessageCoverPercentage = 0;
                    if(accountResp.length > 0) {
                        dash.hcpOptInPercentage = (100 * targetAccountsWithActiveOptInConsent.length / accountResp.length).toFixed();
                        dash.hcpSentMessageCoverPercentage = (100 * targetAccountsWithSentMessage.length / accountResp.length).toFixed();
                    }
                    $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                        name: "OPT_IN_RATE",
                        data: dash.hcpOptInPercentage
                    });
                    $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                        name: "SENT_MESSAGE_COVERAGE",
                        data: dash.hcpSentMessageCoverPercentage
                    });
                    dash.consentReady = true;
                });
            });
        }

        dash.globalFilterOnChange = function() {
            var filterDates = getFilterDates();
            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(filterDates);
            }
            if(dashboardHelper.okToEnableTile(appConstants.CONSENT, _unreachableObjects)) {
                getConsents(filterDates);
            }
        };

        dash.toggleDetailMode = function() {
            dash.detailMode = !dash.detailMode;
            if(!dash.detailMode) {
                dash.showSentMessageDetail = false;
                dash.showConsentDetail = false;
            }
        };

        dash.viewAllSentMessages = function() {
            var fieldLabels = FIELD_LABELS.Sent_Message_vod__c;
            dash.sentMessageTableHeaders = [{
                label: fieldLabels.Content_vod__c,
                name: "Content_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Sent_Datetime_vod__c,
                name: "Sent_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }, {
                label: translationService.getMessage('OPENED'),
                name: "Status",
                isDate: false
            }, {
                label: fieldLabels.Last_Open_Datetime_vod__c,
                name: "Last_Open_Datetime_vod__c",
                isDate: true
            }, {
                label: fieldLabels.Total_Duration_vod__c,
                name: "Total_Duration_vod__c",
                isDate: false
            }, {
                label: translationService.getMessage('SERVICE_ACCOUNT'),
                name: "Service_Account_vod_del__c",
                isDate: false
            }];

            dash.sentMessageTableBody = [];
            dash.sentMessageResp.forEach(function(sm) {
                dash.sentMessageTableBody.push({
                    Content_vod__c: sm.contentName,
                    Account_vod__c: sm.accountName,
                    Sent_Datetime_vod__c: utility.parseStringToDateValue(sm.Sent_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT),
                    Status: sm.Last_Open_Datetime_vod__c ? translationService.getMessage("OPENED") : translationService.getMessage("UNOPENED"),
                    Last_Open_Datetime_vod__c: utility.parseStringToDateValue(sm.Last_Open_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT),
                    Total_Duration_vod__c: sm.Total_Duration_vod__c,
                    Service_Account_vod_del__c: sm.serviceAccountName
                });
            });

            dash.sentMessageObjectLabel = OBJECT_LABELS.Sent_Message_vod__c.plural;
            dash.showSentMessageDetail = true;
            dash.toggleDetailMode();
        };

        dash.viewAllConsents = function() {
            var fieldLabels = FIELD_LABELS.Multichannel_Consent_vod__c;
            dash.consentTableHeaders = [{
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Opt_Type_vod__c,
                name: "Opt_Type_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Capture_Datetime_vod__c,
                name: "Capture_Datetime_vod__c",
                isDate: true
            }, {
                label: fieldLabels.Consent_Confirm_Datetime_vod__c,
                name: "Consent_Confirm_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }];
            dash.consentTableBody = [];
            dash.consentResp.forEach(function(c) {
                dash.consentTableBody.push({
                    Account_vod__c: c.accountName,
                    Opt_Type_vod__c: PICKLIST_LABELS.Multichannel_Consent_vod__c.Opt_Type_vod__c[c.Opt_Type_vod__c],
                    Capture_Datetime_vod__c: utility.parseStringToDateValue(c.Capture_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT),
                    Consent_Confirm_Datetime_vod__c: utility.parseStringToDateValue(c.Consent_Confirm_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT)
                });
            });

            dash.consentObjectLabel = OBJECT_LABELS.Multichannel_Consent_vod__c.plural;
            dash.showConsentDetail = true;
            dash.toggleDetailMode();
        };

        // register ChartJS custom plug-ins
        Chart.pluginService.register(chartJsPlugins.centerLabel);
    }
}());